import os
import inspect
import torch
import numpy as np
from functools import wraps


def auto_generate_data(skip=False):

    def decorator(fn):

        @wraps(fn)
        def wrapper(*args, **kwargs):
            frame = inspect.currentframe().f_back
            file_path = frame.f_code.co_filename
            case_path, _ = os.path.splitext(file_path)

            if skip:
                print(f'{file_path} is skip.')
                return

            #set default random seed
            torch.manual_seed(20240719)
            np.random.seed(20240719)
            inputs, outputs = fn(*args, **kwargs)
            generate_data(case_path, inputs, outputs)

        return wrapper
    
    return decorator


def generate_data(case_path, inputs, outputs):
    if not isinstance(case_path, str):
        raise ValueError(f'case_path must be str, but got {type(case_path)}')
    if not isinstance(inputs, list):
        raise ValueError(f'inputs must be list, but got {type(inputs)}')
    if not isinstance(outputs, list):
        raise ValueError(f'outputs must be list, but got {type(outputs)}')

    for idx, data in enumerate(inputs):
        file_name = case_path + '_input' + str(idx) + '.npy'
        np.save(file_name, data)
    for idx, data in enumerate(outputs):
        file_name = case_path + '_output' + str(idx) + '.npy'
        np.save(file_name, data)
